/*======================================================================
 cgkit - Python Computer Graphics Kit
 Copyright (C) 2004 Matthias Baas (baas@ira.uka.de)

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public
 License along with this library; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 cgkit homepage: http://cgkit.sourceforge.net
======================================================================*/

#define DLL_EXPORT_SLOT
#include "slot.h"

namespace support3d {

/** This counter is used to track the number of allocated/deallocated slot objects
   The counter is increased in the ISlot constructor and decreased in the ISlot destructor.
 */
long ISlot::_slot_counter = 0;

ISlot::ISlot() 
{ 
//  std::cout<<"0x"<<std::hex<<(long)this<<std::dec<<": ISlot<T>::ISlot()"<<std::endl;
  _slot_counter+=1; 
}

ISlot::~ISlot()
{ 
//  std::cout<<"0x"<<std::hex<<(long)this<<std::dec<<": ISlot<T>::~ISlot() begin"<<std::endl;
  _slot_counter-=1; 
  if (_slot_counter<0)
  {
    std::cerr<<"BUG-WARNING: _slot_counter is below zero ("<<_slot_counter<<")!"<<std::endl;
  }
//  std::cout<<"0x"<<std::hex<<(long)this<<std::dec<<": ISlot<T>::~ISlot() end"<<std::endl;
}

}  // end of namespace
