/*
  Main wrapper file. 
 */

#include <boost/python.hpp>
#include <iostream>

#include "vec3.h"
#include "slot.h"
#include "py_slot.h"
#include "boost/shared_ptr.hpp"
#include "worldobject.h"
#include "py_exceptions.h"
#include "debuginfo.h"

#include "component.h"
/*namespace {
#include <windows.h>
#include <GL/gl.h>
}*/

#ifdef WIN32
// for FPU state control
#include <float.h>
#endif

using namespace boost::python;
using namespace support3d;

// noises
void def_noises();

// py_vec3
void class_vec3();
// py_vec4
void class_vec4();
// py_mat3
void class_mat3();
// py_mat4
void class_mat4();
// py_quat
void class_quat();

// py_slots
void class_Slots();
void class_Slots2();
void class_Slots3();
void class_Slots4();
void class_ArraySlots();
void class_ArraySlots2();
void class_Mat3ArraySlot();
void class_Mat4ArraySlot();
void class_ArraySlots4();

// py_boundingbox
void class_BoundingBox();

// py_component
void class_Component();

// py_worldobject
void class_WorldObject();

// py_worldobject3
void class_WorldObjectSlots();

// py_material
void class_Material();

// py_glmaterial
void class_GLMaterial();

// py_geoms
void class_Geoms();

// py_spheregeom
void class_SphereGeom();

// py_ccylindergeom
void class_CCylinderGeom();

// py_torusgeom
void class_TorusGeom();

// py_boxgeom
void class_BoxGeom();

// py_planegeom
void class_PlaneGeom();

// py_trimeshgeom
void class_TriMeshGeom();

// py_polyhedrongeom
void class_PolyhedronGeom();

// py_drawgeom
void class_DrawGeom();

// py_lightsource
void class_LightSource();

// py_glpointlight
void class_GLPointLight();

// py_glspotlight
void class_GLSpotLight();

// py_gldistantlight
void class_GLDistantLight();

// py_glrenderer
void class_GLRenderInstance();


// rply
void rply_read();
void rply_write();

// lib3ds
#ifdef LIB3DS_AVAILABLE
void lib3ds();
#endif

// cyberx3d
#ifdef CYBERX3D_AVAILABLE
void cyberx3d();
#endif

// ogre
#ifdef OGRE_AVAILABLE
void class_OgreCore();
#endif

/**
 */
void StopIterationTranslator(const StopIteration& exc) 
{ 
  PyErr_SetString(PyExc_StopIteration, ""); 
} 

/**
 */
void MemoryErrorTranslator(const EMemoryError& exc) 
{ 
  PyErr_SetString(PyExc_MemoryError, exc.what()); 
} 

/**
 */
void IOErrorTranslator(const EIOError& exc) 
{ 
  PyErr_SetString(PyExc_IOError, exc.what()); 
} 

/**
 */
void RuntimeErrorTranslator(const ERuntimeError& exc) 
{ 
  PyErr_SetString(PyExc_RuntimeError, exc.what()); 
} 

/**
 */
void PyIndexErrorTranslator(const EPyIndexError& exc) 
{ 
  PyErr_SetString(PyExc_IndexError, exc.what()); 
} 

/**
 */
void IndexErrorTranslator(const EIndexError& exc) 
{ 
  PyErr_SetString(PyExc_IndexError, exc.what()); 
} 

/**
 */
void ZeroDivisionErrorTranslator(const EZeroDivisionError& exc) 
{ 
  PyErr_SetString(PyExc_ZeroDivisionError, exc.what()); 
} 

/**
 */
void ValueErrorTranslator(const EValueError& exc) 
{ 
  PyErr_SetString(PyExc_ValueError, exc.what()); 
} 

/**
 */
void KeyErrorTranslator(const EKeyError& exc) 
{ 
  PyErr_SetString(PyExc_KeyError, exc.what()); 
} 

/**
 */
void NotImplementedErrorTranslator(const ENotImplementedError& exc) 
{ 
  PyErr_SetString(PyExc_NotImplementedError, exc.what()); 
} 


/**
  Translator function for the EIncompatibleSlotTypes exception.

  The exception is translated into a ValueError exception.
 */
void IncompatibleSlotTypesTranslator(const EIncompatibleSlotTypes& exc) 
{ 
  //  PyErr_SetString(PyExc_ValueError, "Incompatible slot types"); 
  PyErr_SetString(PyExc_ValueError, exc.what()); 
} 

/**
  Translator function for the ENoInputConnectionsAllowed exception.

  The exception is translated into a ValueError exception.
 */
void NoInputConnectionsAllowedTypesTranslator(const ENoInputConnectionsAllowed& exc) 
{ 
  //  PyErr_SetString(PyExc_ValueError, "Incompatible slot types"); 
  PyErr_SetString(PyExc_ValueError, exc.what()); 
} 

//////////////////////////////////////////////////////////////////////

double _setEpsilon(double eps)
{
  double res = vec3d::epsilon;
  vec3d::epsilon = eps;
  return res;
}

double _getEpsilon()
{
  return vec3d::epsilon;
}

#ifdef WIN32
int getFPUControlState()
{
  return _controlfp(0,0);
}

void setFPUPrecision(int n)
{
  switch(n)
  {
   case 0: _controlfp(_PC_24, _MCW_PC); break;
   case 1: _controlfp(_PC_53, _MCW_PC); break;
   case 2: _controlfp(_PC_64, _MCW_PC); break;
  }
}
#endif

//////////////////////////////////////////////////////////////////////

void foo()
{
}

//////////////////////////////////////////////////////////////////////

BOOST_PYTHON_MODULE(_core)
{
  register_exception_translator<StopIteration>(&StopIterationTranslator);
  register_exception_translator<EValueError>(&ValueErrorTranslator);
  register_exception_translator<EKeyError>(&KeyErrorTranslator);
  register_exception_translator<EPyIndexError>(&PyIndexErrorTranslator);
  register_exception_translator<EMemoryError>(&MemoryErrorTranslator);
  register_exception_translator<EIndexError>(&IndexErrorTranslator);
  register_exception_translator<EIOError>(&IOErrorTranslator);
  register_exception_translator<ERuntimeError>(&RuntimeErrorTranslator);
  register_exception_translator<EIncompatibleSlotTypes>(&IncompatibleSlotTypesTranslator);
  register_exception_translator<ENoInputConnectionsAllowed>(&NoInputConnectionsAllowedTypesTranslator);
  register_exception_translator<EZeroDivisionError>(&ZeroDivisionErrorTranslator);
  register_exception_translator<ENotImplementedError>(&NotImplementedErrorTranslator);
  
  // noise
  def_noises();
  
  // cgtypes
  class_vec3();
  class_vec4();
  class_mat3();
  class_mat4();
  class_quat();
  
  // slot
  class_Slots();
  class_Slots2();
  class_Slots3();
  class_Slots4();
  class_ArraySlots();
  class_ArraySlots2();
  class_Mat3ArraySlot();
  class_Mat4ArraySlot();
  class_ArraySlots4();

  // boundingbox
  class_BoundingBox();

  // component
  class_Component();

  // worldobject
  class_WorldObjectSlots();
  class_WorldObject();

  // material
  class_Material();

  // glmaterial
  class_GLMaterial();

  // geoms
  class_Geoms();

  // SphereGeom
  class_SphereGeom();

  // CCylinderGeom
  class_CCylinderGeom();

  // TorusGeom
  class_TorusGeom();

  // BoxGeom
  class_BoxGeom();

  // PlaneGeom
  class_PlaneGeom();

  // TriMeshGeom
  class_TriMeshGeom();

  // PolyhedronGeom
  class_PolyhedronGeom();

  // DrawGeom
  class_DrawGeom();

  // LightSource
  class_LightSource();

  // GLPointLight
  class_GLPointLight();

  // GLSpotLight
  class_GLSpotLight();

  // GLDistantLight
  class_GLDistantLight();

  // GLRenderInstance
  class_GLRenderInstance();


  //  implicitly_convertible<ArraySlot<int>,ArraySlotWrapper<int> >();
  //  implicitly_convertible<ArraySlotWrapper<int>,ArraySlot<int> >();

  // rply
  rply_read();
  rply_write();

  #ifdef LIB3DS_AVAILABLE
  lib3ds();
  #endif

  #ifdef CYBERX3D_AVAILABLE
  cyberx3d();
  #endif
  
  #ifdef OGRE_AVAILABLE
  class_OgreCore();
  #endif

  // Test
  def("_set_debug_flag", set_debug_flag);
  def("_get_debug_flag", get_debug_flag);
  def("_setEpsilon", _setEpsilon);
  def("_getEpsilon", _getEpsilon);
  def("foo", &foo);
  //  def("drawGL_test", &drawGL_test);

  #ifdef WIN32
  def("getFPUControlState", getFPUControlState);
  def("setFPUPrecision", setFPUPrecision);
  #endif
}
